@@ -6,7 +6,7 @@ Requires Python 2.1 or newer.
 
 This file contains all the classes and helper functions specific to IPython.
 
-$Id: iplib.py,v 1.90 2005/04/08 08:38:42 fperez Exp $
+$Id: iplib.py,v 1.95 2005/06/01 17:01:13 fperez Exp $
 """
 
 #*****************************************************************************
@@ -224,7 +224,8 @@ try:
                 self.clean_glob = self._clean_glob
             self.matchers = [self.python_matches,
                              self.file_matches,
-                             self.alias_matches]
+                             self.alias_matches,
+                             self.python_func_kw_matches]
 
         # Code contributed by Alex Schmolck, for ipython/emacs integration
         def all_completions(self, text):
@@ -279,15 +280,23 @@ try:
                                 for ch in s])
 
             lbuf = self.get_line_buffer()[:self.readline.get_endidx()]
+            open_quotes = 0  # track strings with open quotes
             try:
                 lsplit = shlex_split(lbuf)[-1]
             except ValueError:
-                # unmatched ", or backslash without escaped character typically
-                return None   
+                # typically an unmatched ", or backslash without escaped char.
+                if lbuf.count('"')==1:
+                    open_quotes = 1
+                    lsplit = lbuf.split('"')[-1]
+                elif lbuf.count("'")==1:
+                    open_quotes = 1
+                    lsplit = lbuf.split("'")[-1]
+                else:
+                    return None
             except IndexError:
                 # tab pressed on empty line
                 lsplit = ""
-                
+
             if lsplit != protect_filename(lsplit):
                 # if protectables are found, do matching on the whole escaped
                 # name
@@ -308,13 +317,18 @@ try:
                 len_lsplit = len(lsplit)
                 matches = [text0 + protect_filename(f[len_lsplit:]) for f in m0]
             else:
-                matches = [protect_filename(f) for f in m0]
+                if open_quotes:
+                    # if we have a string with an open quote, we don't need to
+                    # protect the names at all (and we _shouldn't_, as it
+                    # would cause bugs when the filesystem call is made).
+                    matches = m0
+                else:
+                    matches = [protect_filename(f) for f in m0]
             if len(matches) == 1 and os.path.isdir(matches[0]):
                 # Takes care of links to directories also.  Use '/'
                 # explicitly, even under Windows, so that name completions
                 # don't end up escaped.
                 matches[0] += '/'
-
             return matches
 
         def alias_matches(self, text):
@@ -353,6 +367,82 @@ try:
                     matches = self.attr_matches(self.magic_prefix+text)
             return matches
 
+        def _default_arguments(self, obj):
+            """Return the list of default arguments of obj if it is callable,
+            or empty list otherwise."""
+            
+            if not (inspect.isfunction(obj) or inspect.ismethod(obj)):
+                # for classes, check for __init__,__new__
+                if inspect.isclass(obj):
+                    obj = (getattr(obj,'__init__',None) or
+                           getattr(obj,'__new__',None))
+                # for all others, check if they are __call__able
+                elif hasattr(obj, '__call__'):
+                    obj = obj.__call__
+                # XXX: is there a way to handle the builtins ?
+            try:
+                args,_,_1,defaults = inspect.getargspec(obj)
+                if defaults:
+                    return args[-len(defaults):]
+            except TypeError: pass
+            return []
+
+        def python_func_kw_matches(self,text):
+            """Match named parameters (kwargs) of the last open function"""
+
+            if "." in text: # a parameter cannot be dotted
+                return []
+            try: regexp = self.__funcParamsRegex
+            except AttributeError:
+                regexp = self.__funcParamsRegex = re.compile(r'''
+                    '.*?' |    # single quoted strings or
+                    ".*?" |    # double quoted strings or
+                    \w+   |    # identifier
+                    \S         # other characters
+                    ''', re.VERBOSE | re.DOTALL)
+            # 1. find the nearest identifier that comes before an unclosed
+            # parenthesis e.g. for "foo (1+bar(x), pa", the candidate is "foo"
+            tokens = regexp.findall(self.get_line_buffer())
+            tokens.reverse()
+            iterTokens = iter(tokens); openPar = 0
+            for token in iterTokens:
+                if token == ')':
+                    openPar -= 1
+                elif token == '(':
+                    openPar += 1
+                    if openPar > 0:
+                        # found the last unclosed parenthesis
+                        break
+            else:
+                return []
+            # 2. Concatenate any dotted names (e.g. "foo.bar" for "foo.bar(x, pa" )
+            ids = []
+            isId = re.compile(r'\w+$').match
+            while True:
+                try:
+                    ids.append(iterTokens.next())
+                    if not isId(ids[-1]):
+                        ids.pop(); break
+                    if not iterTokens.next() == '.':
+                        break
+                except StopIteration:
+                    break
+            # lookup the candidate callable matches either using global_matches
+            # or attr_matches for dotted names
+            if len(ids) == 1:
+                callableMatches = self.global_matches(ids[0])
+            else:
+                callableMatches = self.attr_matches('.'.join(ids[::-1]))
+            argMatches = []
+            for callableMatch in callableMatches:
+                try: namedArgs = self._default_arguments(eval(callableMatch,
+                                                             self.namespace))
+                except: continue
+                for namedArg in namedArgs:
+                    if namedArg.startswith(text):
+                        argMatches.append("%s=" %namedArg)
+            return argMatches
+
         def complete(self, text, state):
             """Return the next possible completion for 'text'.
 
@@ -382,7 +472,6 @@ try:
                             if self.matches:
                                 break
                         
-                    #print 'matches:',self.matches  # dbg
                 try:
                     return self.matches[state].replace(magic_prefix,magic_escape)
                 except IndexError:
@@ -555,11 +644,13 @@ class InteractiveShell(code.InteractiveConsole, Logger, Magic):
         self.ESC_HELP  = '?'
         self.ESC_MAGIC = '%'
         self.ESC_QUOTE = ','
+        self.ESC_QUOTE2 = ';'
         self.ESC_PAREN = '/'
 
         # And their associated handlers
         self.esc_handlers = {self.ESC_PAREN:self.handle_auto,
                              self.ESC_QUOTE:self.handle_auto,
+                             self.ESC_QUOTE2:self.handle_auto,
                              self.ESC_MAGIC:self.handle_magic,
                              self.ESC_HELP:self.handle_help,
                              self.ESC_SHELL:self.handle_shell_escape,
@@ -646,7 +737,7 @@ class InteractiveShell(code.InteractiveConsole, Logger, Magic):
         # much:  it's better to be conservative rather than to trigger hidden
         # evals() somewhere and end up causing side effects.
 
-        self.line_split = re.compile(r'^(\s*)'
+        self.line_split = re.compile(r'^([\s*,;/])'
                                      r'([\?\w\.]+\w*\s*)'
                                      r'(\(?.*$)')
 
@@ -878,7 +969,10 @@ class InteractiveShell(code.InteractiveConsole, Logger, Magic):
         and 'upgrade'."""
 
         def wait():
-            raw_input("Please press <RETURN> to start IPython.")
+            try:
+                raw_input("Please press <RETURN> to start IPython.")
+            except EOFError:
+                print >> Term.cout
             print '*'*70
 
         cwd = os.getcwd()  # remember where we started
@@ -947,8 +1041,15 @@ Can not upgrade: changing to directory %s failed. Details:
                     new_filename = os.path.basename(new_full_path)
                     if new_filename.startswith('ipythonrc'):
                         new_filename = new_filename + rc_suffix
+                    # The config directory should only contain files, skip any
+                    # directories which may be there (like CVS)
+                    if os.path.isdir(new_full_path):
+                        continue
                     if os.path.exists(new_filename):
-                        os.rename(new_filename,new_filename+'.old')
+                        old_file = new_filename+'.old'
+                        if os.path.exists(old_file):
+                            os.remove(old_file)
+                        os.rename(new_filename,old_file)
                     shutil.copy(new_full_path,new_filename)
         else:
             raise ValueError,'unrecognized mode for install:',`mode`
@@ -1792,13 +1893,18 @@ There seemed to be a problem with your sys.stderr.
                     pre=None,iFun=None,theRest=None):
         """Hande lines which can be auto-executed, quoting if requested."""
 
+        #print 'pre <%s> iFun <%s> rest <%s>' % (pre,iFun,theRest)  # dbg
+        
         # This should only be active for single-line input!
         if continue_prompt:
             return line
 
         if pre == self.ESC_QUOTE:
-            # Auto-quote
+            # Auto-quote splitting on whitespace
             newcmd = '%s("%s")\n' % (iFun,'", "'.join(theRest.split()) )
+        elif pre == self.ESC_QUOTE2:
+            # Auto-quote whole string
+            newcmd = '%s("%s")\n' % (iFun,theRest)
         else:
             # Auto-paren
             if theRest[0:1] in ('=','['):
@@ -1872,16 +1978,16 @@ There seemed to be a problem with your sys.stderr.
         try:
             xfile = open(fname)
         except:
-            print >> sys.stderr, \
+            print >> Term.cerr, \
                   'Could not open file <%s> for safe execution.' % fname
             return None
 
         kw.setdefault('islog',0)
         kw.setdefault('quiet',1)
+        kw.setdefault('exit_ignore',0)
         first = xfile.readline()
         _LOGHEAD = str(self.LOGHEAD).split('\n',1)[0].strip()
         xfile.close()
-        xfile = open(fname)
         # line by line execution
         if first.startswith(_LOGHEAD) or kw['islog']:
             print 'Loading log file <%s> one line at a time...' % fname
@@ -1904,6 +2010,7 @@ There seemed to be a problem with your sys.stderr.
             # first, and manually walk through the lines list moving the
             # counter ourselves.
             indent_re = re.compile('\s+\S')
+            xfile = open(fname)
             filelines = xfile.readlines()
             xfile.close()
             nlines = len(filelines)
@@ -1955,9 +2062,13 @@ There seemed to be a problem with your sys.stderr.
             try:
                 execfile(fname,*where)
             except SyntaxError:
-                type, value = sys.exc_info()[0:2]
-                self.SyntaxTB(type,value,[])
+                etype, evalue = sys.exc_info()[0:2]
+                self.SyntaxTB(etype,evalue,[])
                 warn('Failure executing file: <%s>' % fname)
+            except SystemExit,status:
+                if not kw['exit_ignore']:
+                    self.InteractiveTB()
+                    warn('Failure executing file: <%s>' % fname)
             except:
                 self.InteractiveTB()
                 warn('Failure executing file: <%s>' % fname)
