@@ -6,7 +6,7 @@ Requires Python 2.1 or better.
 
 This file contains the main make_IPython() starter function.
 
-$Id: ipmaker.py,v 1.32 2005/04/07 00:09:05 fperez Exp $"""
+$Id: ipmaker.py,v 1.33 2005/05/13 21:20:00 fperez Exp $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2004 Fernando Perez. <fperez@colorado.edu>
@@ -173,7 +173,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     # The "ignore" option is a kludge so that Emacs buffers don't crash, since
     # the 'C-c !' command in emacs automatically appends a -i option at the end.
     cmdline_only = ('help ignore|i ipythondir=s Version upgrade '
-                    'gthread! wthread! pylab! tk!')
+                    'gthread! qthread! wthread! pylab! tk!')
 
     # Build the actual name list to be used by DPyGetOpt
     opts_names = qw(cmdline_opts) + qw(cmdline_only)
@@ -220,6 +220,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                       separate_out2 = '',
                       system_verbose = 0,
                       gthread = 0,
+                      qthread = 0,
                       wthread = 0,
                       pylab = 0,
                       tk = 0,
