@@ -60,7 +60,7 @@ You can implement other color schemes easily, the syntax is fairly
 self-explanatory. Please send back new schemes you develop to the author for
 possible inclusion in future releases.
 
-$Id: ultraTB.py,v 1.25 2005/01/18 19:19:39 fperez Exp $"""
+$Id: ultraTB.py,v 1.27 2005/05/30 06:26:28 fperez Exp $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001 Nathaniel Gray <n8gray@caltech.edu>
@@ -460,11 +460,17 @@ class VerboseTB(TBTools):
                                             ColorsNormal)
 
         # now, loop over all records printing context and info
+        abspath = os.path.abspath
         for frame, file, lnum, func, lines, index in records:
             #print '*** record:',file,lnum,func,lines,index  # dbg
-            file = file and os.path.abspath(file) or '?'
+            try:
+                file = file and abspath(file) or '?'
+            except OSError:
+                # if file is '<console>' or something not in the filesystem,
+                # the abspath call will throw an OSError.  Just ignore it and
+                # keep the original file string.
+                pass
             link = tpl_link % file
-
             try:
                 args, varargs, varkw, locals = inspect.getargvalues(frame)
             except:
@@ -552,6 +558,11 @@ class VerboseTB(TBTools):
             except IndexError:
                 # signals exit of tokenizer
                 pass
+            except tokenize.TokenError,msg:
+                _m = ("An unexpected error occurred while tokenizing input\n"
+                      "The following traceback may be corrupted or invalid\n"
+                      "The error message is: %s\n" % msg)
+                error(_m)
             
             # prune names list of duplicates, but keep the right order
             unique_names = uniq_stable(names)
