@@ -6,7 +6,7 @@
 #  the file COPYING, distributed as part of this software.
 #*****************************************************************************
 
-# $Id: usage.py,v 1.21 2005/03/02 07:10:06 fperez Exp $
+# $Id: usage.py,v 1.22 2005/05/13 21:16:51 fperez Exp $
 
 from IPython import Release
 __author__  = '%s <%s>' % Release.authors['Fernando']
@@ -53,46 +53,46 @@ SPECIAL THREADING OPTIONS
        ization of ipython itself, before the normal option-handling  mechanism
        is active.
 
-       -gthread, -wthread, -pylab
-              Only  ONE of these can be given, and it can only be given as the
-              first option passed to IPython (it will have no  effect  in  any
-              other position).  They provide threading support for the GTK and
-              WXPython toolkits, and for the matplotlib library.
+       -gthread, -qthread, -wthread, -pylab
 
-              If -gthread is given, IPython starts running a  separate  thread
-              for  GTK  operation,  so  that pyGTK-based programs can open and
-              control GUIs  without  blocking  IPython.
+              Only ONE of these can be given, and it can only be given as the
+              first option passed to IPython (it will have no effect in any
+              other position).  They provide threading support for the GTK, QT
+              and WXWidgets toolkits, and for the matplotlib library.
 
-              Similarly, -wthread instantiates IPython with threading support
-              for the WXPython toolkit.  You can control WX application
-              windows from within IPython.
+              With any of the first three options, IPython starts running a
+              separate thread for the graphical toolkit's operation, so that
+              you can open and control graphical elements from within an
+              IPython command line, without blocking.  All three provide
+              essentially the same functionality, respectively for GTK, QT and
+              WXWidgets (via their Python interfaces).
 
-              If  -pylab  is given, IPython loads special support for the mat-
-              plotlib  library  (http://matplotlib.sourceforge.net),  allowing
-              interactive  usage  of  any  of  its  backends as defined in the
-              user’s .matplotlibrc file.  It automatically activates GTK or WX
-              threading  for  IPyhton  if  the  choice  of  matplotlib backend
-              requires it.  It also modifies the  %run  command  to  correctly
-              execute  (without  blocking)  any  matplotlib-based script which
+              If -pylab is given, IPython loads special support for the mat-
+              plotlib library (http://matplotlib.sourceforge.net), allowing
+              interactive usage of any of its backends as defined in the
+              user's .matplotlibrc file.  It automatically activates GTK, QT
+              or WX threading for IPyhton if the choice of matplotlib backend
+              requires it.  It also modifies the %run command to correctly
+              execute (without blocking) any matplotlib-based script which
               calls show() at the end.
 
-       -tk    The -g/wthread options, and -pylab (if matplotlib is  configured
-              to  use WX or GTK), will normally block Tk graphical interfaces.
-              This means that when either GTK or WX threading is  active,  any
-              attempt  to open a Tk GUI will result in a dead window, and pos-
-              sibly cause the Python interpreter to crash.  An  extra  option,
-              -tk,  is  available to address this issue.  It can ONLY be given
-              as a SECOND option after any of the above (-gthread, -wthread or
-              -pylab).
+       -tk    The -g/q/wthread options, and -pylab (if matplotlib is
+              configured to use GTK, QT or WX), will normally block Tk
+              graphical interfaces.  This means that when GTK, QT or WX
+              threading is active, any attempt to open a Tk GUI will result in
+              a dead window, and possibly cause the Python interpreter to
+              crash.  An extra option, -tk, is available to address this
+              issue.  It can ONLY be given as a SECOND option after any of the
+              above (-gthread, -qthread, -wthread or -pylab).
 
               If -tk is given, IPython will try to coordinate Tk threading
-              with WX or GTK.  This is however potentially unreliable, and you
-              will have to test on your platform and Python configuration to
-              determine whether it works for you.  Debian users have reported
-              success, apparently due to the fact that Debian builds all of
-              Tcl, Tk, Tkinter and Python with pthreads support.  Under other
-              Linux environments (such as Fedora Core 2), this option has
-              caused random crashes and lockups of the Python interpreter.
+              with GTK, QT or WX.  This is however potentially unreliable, and
+              you will have to test on your platform and Python configuration
+              to determine whether it works for you.  Debian users have
+              reported success, apparently due to the fact that Debian builds
+              all of Tcl, Tk, Tkinter and Python with pthreads support.  Under
+              other Linux environments (such as Fedora Core 2/3), this option
+              has caused random crashes and lockups of the Python interpreter.
               Under other operating systems (Mac OSX and Windows), you'll need
               to try it to find out, since currently no user reports are
               available.
@@ -101,6 +101,23 @@ SPECIAL THREADING OPTIONS
               time  whether -tk will work reliably or not, so you will need to
               do some experiments before relying on it for regular work.
 
+A WARNING ABOUT SIGNALS AND THREADS
+
+       When any of the thread systems (GTK, QT or WX) are active, either
+       directly or via -pylab with a threaded backend, it is impossible to
+       interrupt long-running Python code via Ctrl-C.  IPython can not pass
+       the KeyboardInterrupt exception (or the underlying SIGINT) across
+       threads, so any long-running process started from IPython will run to
+       completion, or will have to be killed via an external (OS-based)
+       mechanism.
+
+       To the best of my knowledge, this limitation is imposed by the Python
+       interpreter itself, and it comes from the difficulty of writing
+       portable signal/threaded code.  If any user is an expert on this topic
+       and can suggest a better solution, I would love to hear about it.  In
+       the IPython sources, look at the Shell.py module, and in particular at
+       the runcode() method.
+
 REGULAR OPTIONS
        After the above threading options have been given, regular options  can
        follow  in any order.  All options can be abbreviated to their shortest
@@ -548,29 +565,22 @@ MAIN FEATURES
             ------> zip ((1,2,3),(4,5,6))
             Out[2]= [(1, 4), (2, 5), (3, 6)]        
 
-    2. Auto-Quoting
-        You can force auto-quoting of a function's arguments by using ',' as
-        the first character of a line.  For example:
-            >>> ,my_function /home/me   # becomes my_function("/home/me")
-        Note that the ',' MUST be the first character on the line!  This
-        won't work:
-            >>> x = ,my_function /home/me    # syntax error
-
-  Notes on usage of these two features:
-  
-    1.  IPython tells you that it has altered your command line by
+        IPython tells you that it has altered your command line by
         displaying the new command line preceded by -->.  e.g.:
             In [18]: callable list
             -------> callable (list) 
 
-    2.  Whitespace is more important than usual (even for Python!)
-        Arguments to auto-quote functions cannot have embedded whitespace.
-        
-            In [21]: ,string.split a b
-            -------> string.split ("a", "b")
-            Out[21]= ['a']       # probably not what you wanted
+    2. Auto-Quoting
+        You can force auto-quoting of a function's arguments by using ',' as
+        the first character of a line.  For example:
+            >>> ,my_function /home/me   # becomes my_function("/home/me")
 
-            In [22]: string.split 'a b'
-            -------> string.split ('a b')
-            Out[22]= ['a', 'b']  # quote explicitly and it works.
+        If you use ';' instead, the whole argument is quoted as a single
+        string (while ',' splits on whitespace):
+            >>> ,my_function a b c   # becomes my_function("a","b","c")
+            >>> ;my_function a b c   # becomes my_function("a b c")
+
+        Note that the ',' MUST be the first character on the line!  This
+        won't work:
+            >>> x = ,my_function /home/me    # syntax error
 """
